import peasy.test.*;
import peasy.org.apache.commons.math.*;
import peasy.*;
import peasy.org.apache.commons.math.geometry.*;

int deweyScale = 13;
int monthScale = 42;
int loanScale = 13;
int shapeScale = 4;
int camDist = 800;

PeasyCam cam;
PShape books;
PShape baseBooks;
PShape axis;
PShape origin;
PShape focus;
PImage tex;
PImage bg;
int[][] data;
String[] topic;

boolean sketchFullScreen() {
  return true;
}

void setup() {
    ((javax.swing.JFrame)frame).getContentPane().setBackground(new java.awt.Color(0));
    int xMargin = displayWidth % 100;
    int yMargin = displayHeight % 100;
    if (xMargin < 80) xMargin = 80;
    if (yMargin < 60) yMargin = 60;
    size(displayWidth - xMargin, displayHeight - yMargin, P3D);
    noSmooth();
    noLoop();

    Table checkoutTable = loadTable("PhilosophyCheckouts.csv", "header");
    Table topicTable = loadTable("PhilosophyClasses.csv", "header");
    int numRowData = checkoutTable.getRowCount();
    // 3 col: dewey, month, numCheckout
    data = new int[numRowData][4];
    for (int i = 0; i < numRowData; ++i) {
        data[i][0] = checkoutTable.getInt(i, 0);
        data[i][1] = checkoutTable.getInt(i, 1);
        data[i][2] = checkoutTable.getInt(i, 3);
        data[i][3] = checkoutTable.getInt(i, 4);
    }
    int numRowTopic = topicTable.getRowCount();
    topic = new String[numRowTopic];
    for (int i = 0; i < numRowTopic; ++i) {
        topic[i] = topicTable.getString(i, 1);
    }

    books = createShape(GROUP);
    tex = loadImage("book_tex_small.jpg");
    float log2 = log(2);
    for (int i = 0; i < numRowData; ++i) {
        PShape pshape = createShape();
        pshape.beginShape(QUADS);
        pshape.noStroke();
        pshape.texture(tex);
        // front
        pshape.normal(0, 0, 1);
        pshape.vertex(-4, -5, 1, 24, 0);
        pshape.vertex(4, -5, 1, 103, 0);
        pshape.vertex(4, 5, 1, 103, 127);
        pshape.vertex(-4, 5, 1, 24, 127);
        // back
        pshape.normal(0, 0, -1);
        pshape.vertex(-4, -5, -1, 24, 0);
        pshape.vertex(4, -5, -1, 103, 0);
        pshape.vertex(4, 5, -1, 103, 127);
        pshape.vertex(-4, 5, -1, 24, 127);
        // strip
        pshape.normal(-1, 0, 0);
        pshape.vertex(-4, -5, -1, 0, 0);
        pshape.vertex(-4, -5, 1, 24, 0);
        pshape.vertex(-4, 5, 1, 24, 127);
        pshape.vertex(-4, 5, -1, 0, 127);
        // top
        pshape.normal(0, -1, 0);
        pshape.vertex(-4, -5, 1, 103, 0);
        pshape.vertex(-4, -5, -1, 127, 0);
        pshape.vertex(4, -5, -1, 127, 127);
        pshape.vertex(4, -5, 1, 103, 127);
        // bottom
        pshape.normal(0, 1, 0);
        pshape.vertex(-4, 5, 1, 103, 0);
        pshape.vertex(-4, 5, -1, 127, 0);
        pshape.vertex(4, 5, -1, 127, 127);
        pshape.vertex(4, 5, 1, 103, 127);
        // side
        pshape.normal(1, 0, 0);
        pshape.vertex(4, -5, -1, 103, 0);
        pshape.vertex(4, -5, 1, 127, 0);
        pshape.vertex(4, 5, 1, 127, 127);
        pshape.vertex(4, 5, -1, 103, 127);
        pshape.endShape();

        // -100 for dewey since it starts at 100, same for month
        // log scale the loan time
        // add little bit of displacement
        float transX = (data[i][0] - 100) * deweyScale;
        float transY = (data[i][1] - 1) * monthScale;
        float transZ = log(data[i][2]) / log2 * loanScale;
        transX += random(-deweyScale/3.0, deweyScale/3.0);
        transY += random(-monthScale/3.0, monthScale/3.0);
        transZ += random(-loanScale/3.0, loanScale/3.0);
        pshape.translate(transX, transY, transZ);
        // shake a bit
        pshape.rotateX(random(-0.1, 0.1) * PI);
        pshape.rotateY(random(-0.1, 0.1) * PI);
        pshape.scale(shapeScale);
        int redTint = 3 * data[i][3];
        pshape.setTint(color(100 + redTint, 255 - redTint, 100, 150 + redTint));
        books.addChild(pshape);
    }

    axis = createShape();
    axis.beginShape(LINES);
    axis.strokeWeight(5);
    axis.stroke(200, 0, 0, 100);
    axis.vertex(height / 40, 0, 0);
    axis.vertex(height / 2, 0, 0);
    axis.stroke(0, 200, 0, 100);
    axis.vertex(0, height / 40, 0);
    axis.vertex(0, height / 2, 0);
    axis.stroke(0, 0, 200, 100);
    axis.vertex(0, 0, height / 40);
    axis.vertex(0, 0, height / 2);
    axis.endShape();

    origin = createShape(BOX, height / 20);
    origin.setStroke(false);
    origin.setFill(color(200, 200, 200, 200));

    focus = createShape(BOX, 20);
    focus.setStroke(false);
    focus.setFill(color(255, 255, 255, 50));

    int xCen = 100 * deweyScale / 2;
    int yCen = 12 * monthScale / 2;
    int zCen = 0;
    cam= new PeasyCam(this, xCen, yCen, zCen, camDist);
    cam.setMinimumDistance(camDist / 4.0);
    cam.setMaximumDistance(camDist * 4.0);

    bg = loadImage("bookshelf.jpg");
    bg.resize(width, height);
    bg.filter(BLUR, 4);

    textFont(loadFont("SourceSansPro-Semibold-48.vlw"));
    textSize(13);
}

void draw() {
    background(bg);
    shape(books);

    for (int i = 0; i < topic.length; ++i) {
        pushMatrix();
        translate(deweyScale * i, -30, 0);
        rotateZ(-0.5 * PI);
        textAlign(LEFT, TOP);
        text(topic[i], 0, 0);
        popMatrix();
    }
    // shape(axis);
    // shape(origin);
    // focus.resetMatrix();
    // float[] camPos = cam.getLookAt();
    // focus.translate(camPos[0], camPos[1], camPos[2]);
    // shape(focus);
}

void mouseDragged() {
    redraw();
}
